from flask import Flask, jsonify
from dotenv import load_dotenv
import psycopg2

from flask_cors import CORS, cross_origin

import datetime
import re

from datagen import generate_data

from datetime import date

# setup environment variables
load_dotenv()
import os
print(os.environ.get('DB_NAME'))

# set up db connection
conn = psycopg2.connect(
    host = os.environ.get('DB_HOST'),
    database = os.environ.get('DB_NAME'),
    user = os.environ.get('DB_USER'),
    password = os.environ.get('DB_PASSWORD'))

app = Flask(__name__)
CORS(app, resources={r"/*": {"origins": "*"}})

def parse_timestamp(timestamp):
  return datetime.datetime(*[int(x) for x in re.findall(r'\d+', timestamp)])

@app.route("/")
def home_route():
    return "<p>Hello, World!</p>"

@app.route('/database')
def database_route():
    return jsonify({"test": "data"})

@app.route('/valid50tweets')
def valid50():
    cur = conn.cursor()
    cur.execute('SELECT * FROM tweets WHERE geo IS NOT NULL AND geo <> %s limit 50', ('',))
    result = cur.fetchall()

    cur.close()
    return jsonify({"tweets":result})

@app.route("/symptoms")
def symptoms_route():
    #determine the 10 most frequent symptoms
    cur1 = conn.cursor()
    cur2 = conn.cursor()
    cur1.execute("select u.symptom from final_tweets t cross join unnest(t.symptoms) as u(symptom) group by u.symptom order by count(*) desc limit 10")
    result = cur1.fetchall()
    #create a dictionary with these symptoms as keys
    symptom_list = [col[0] for col in result]
    data = dict.fromkeys(symptom_list)
    for key in data:
        #for each symptom, get the count of tweets including it per month
        cur2.execute("select extract (epoch from DATE_TRUNC('month', created_at)) AS  month, count(id) as count from final_tweets where %s = any(symptoms) GROUP BY DATE_TRUNC('month',created_at) ORDER BY 1 ASC", (key,))
        symp_data = cur2.fetchall()

        values = []
        for row2 in symp_data:
            values.append(row2[1])
        max_value = max(values)

        symp_array = []
        for row in symp_data:
            #add these values to an array 
            symp_array.append({'timestamp': (int(row[0]) * 1000), 'value': (row[1] * 100)/max_value})
        #assign array to the specific symptom
        data[key] = symp_array
    cur2.close()
    return jsonify(data)

@app.route("/symptoms/<path:subpath>")
def symptoms_route_dates(subpath):
    symptoms = () #need to create tuple of most freq symptoms
    paths = subpath.split('/')
    start_date = date.fromtimestamp(int(paths[0]))
    end_date = date.fromtimestamp(int(paths[1]))
    cur = conn.cursor()
    cur.execute("SELECT DATE_TRUNC('month', created_at) AS month, COUNT(id) as count FROM final_tweets WHERE '%s' = any(symptoms) GROUP BY DATE_TRUNC('month', created_at) ORDER BY 1 ASC", symptoms)
    result = cur.fetchall()
    return jsonify(result)

@app.route("/foods")
def foods_route():
    #determine the 10 most frequent foods
    cur1 = conn.cursor()
    cur2 = conn.cursor()
    cur1.execute("select u.food from final_tweets t cross join unnest(t.foods) as u(food) group by u.food order by count(*) desc limit 10")
    result = cur1.fetchall()
    #create a dictionary with these sfoods as keys
    symptom_list = [col[0] for col in result]
    data = dict.fromkeys(symptom_list)
    for key in data:
        #for each food, get the count of tweets including it per month
        cur2.execute("select extract (epoch from DATE_TRUNC('month', created_at)) AS  month, count(id) as count from final_tweets where %s = any(foods) GROUP BY DATE_TRUNC('month',created_at) ORDER BY 1 ASC", (key,))
        symp_data = cur2.fetchall()
        symp_array = []
        for row in symp_data:
            #add these values to an array 
            symp_array.append({'timestamp': row[0], 'value': row[1]})
        #assign array to the specific food
        data[key] = symp_array
    cur2.close()
    return jsonify(data)

@app.route("/foods/<path:subpath>")
def foods_route_dates(subpath):
    paths = subpath.split('/')
    #start_date = date.fromtimestamp(int(paths[0]))
    #end_date = date.fromtimestamp(int(paths[1]))
    foods = () #need to create tuple of most freq symptoms
    paths = subpath.split('/')
    cur = conn.cursor()
    cur.execute("SELECT DATE_TRUNC('month', created_at) AS month, COUNT(id) as count, FROM final_tweets WHERE '%s' = any(foods) GROUP BY DATE_TRUNC('month', created_at) ORDER BY 1 ASC", foods)
    result = cur.fetchall()

    return jsonify(result)

@app.after_request
def handle_options(response):
    response.headers["Access-Control-Allow-Origin"] = "*"
    response.headers["Access-Control-Allow-Methods"] = "GET, POST, PUT, DELETE, OPTIONS"
    response.headers["Access-Control-Allow-Headers"] = "*"

    return response
